const fs = require('fs');
const tokens = require('../index');
const { namespace } = require('./config');

const distFolder = 'partials';

function prefixName(name, prefix = namespace) {
  return `${prefix}${name}`;
}

function generateColors() {
  return Object.keys(tokens.colors.colorsMap).reduce(
    (output, name) => `${output}.${prefixName(name)} { color: $${prefixName(name)}; }\n`,
    ''
  );
}

function generateTypefaces() {
  return Object.keys(tokens.typography.typeface).reduce(
    (output, name) => `${output}.${prefixName(name)} { font-family: $${prefixName(name)}; }\n`,
    ''
  );
}

function generateWeights() {
  return Object.keys(tokens.typography.weights).reduce(
    (output, name) => `${output}.${prefixName(name)} { font-weight: $${prefixName(name)}; }\n`,
    ''
  );
}

function generateFontSizes() {
  return Object.keys(tokens.typography.fontSizes).reduce(
    (output, name) => `${output}.${prefixName(name)} { font-size: $${prefixName(name)}; }\n`,
    ''
  );
}

function generateLetterSpacing() {
  return Object.keys(tokens.typography.letterSpacings).reduce(
    (output, name) => `${output}.${prefixName(name)} { letter-spacing: $${prefixName(name)}; }\n`,
    ''
  );
}

function generateTypography() {
  return `${generateTypefaces()}${generateWeights()}${generateFontSizes()}${generateLetterSpacing()}`;
}

function generateRadii() {
  return Object.keys(tokens.radii.values).reduce(
    (output, name) => `${output}.${prefixName(name)} { border-radius: $${prefixName(name)}; }\n`,
    ''
  );
}

function generateShadows() {
  return Object.keys(Object.assign({}, tokens.shadows.outer, tokens.shadows.inner, tokens.shadows.innerN)).reduce(
    (output, name) => `${output}.${prefixName(name)} { box-shadow: $${prefixName(name)}; }\n`,
    ''
  );
}

function generateSpacing() {
  const multipliers = tokens.spacing.multipliers;
  const scale = Object.keys(tokens.spacing.scale);
  const types = ['margin', 'padding'];
  const sides = ['top', 'left', 'right', 'bottom'];
  const allSides = types
    .map(type => {
      const typeAbbr = type.substring(0, 1);
      let multiplier = 0;
      return scale.reduce((output, name) => {
        const scaleNum = multipliers[multiplier];
        multiplier += 1;
        return `${output}.${prefixName(`${typeAbbr}-${scaleNum}`)} { ${type}: $${prefixName(name)}; }\n`;
      }, '');
    })
    .join('');
  const perSides = types
    .map(type => {
      const typeAbbr = type.substring(0, 1);
      return sides
        .map(side => {
          const sideAbbr = side.substring(0, 1);
          let multiplier = 0;
          return scale.reduce((output, name) => {
            const scaleNum = multipliers[multiplier];
            multiplier += 1;
            return `${output}.${prefixName(`${typeAbbr}${sideAbbr}-${scaleNum}`)} { ${type}-${side}: $${prefixName(
              name
            )}; }\n`;
          }, '');
        })
        .join('');
    })
    .join('');
  const verticals = types
    .map(type => {
      const typeAbbr = type.substring(0, 1);
      let multiplier = 0;
      return scale.reduce((output, name) => {
        const scaleNum = multipliers[multiplier];
        multiplier += 1;
        return `${output}\n.${prefixName(`${typeAbbr}v-${scaleNum}`)} {
  ${type}-top: $${prefixName(name)};
  ${type}-bottom: $${prefixName(name)};
}\n`;
      }, '');
    })
    .join('');
  const horizontals = types
    .map(type => {
      const typeAbbr = type.substring(0, 1);
      let multiplier = 0;
      return scale.reduce((output, name) => {
        const scaleNum = multipliers[multiplier];
        multiplier += 1;
        return `${output}\n.${prefixName(`${typeAbbr}h-${scaleNum}`)} {
  ${type}-left: $${prefixName(name)};
  ${type}-right: $${prefixName(name)};
}\n`;
      }, '');
    })
    .join('');
  return `${allSides}${perSides}${verticals}${horizontals}`;
}

function generateTransition() {
  return Object.keys(tokens.transition.durations).reduce(
    (output, name) => `${output}.${prefixName(name)} { transition-duration: $${prefixName(name)}; }\n`,
    ''
  );
}

function generateImports() {
  return Object.keys(tokens).reduce((scss, token) => `${scss}@import '${token}.scss';\n`, '');
}

function generateVariablesImport(output) {
  return `@import '../index.scss';\n${output}`;
}

if (!fs.existsSync(distFolder)) fs.mkdirSync(distFolder);
fs.writeFileSync(`${distFolder}/_colors.scss`, generateVariablesImport(generateColors()));
fs.writeFileSync(`${distFolder}/_typography.scss`, generateVariablesImport(generateTypography()));
fs.writeFileSync(`${distFolder}/_radii.scss`, generateVariablesImport(generateRadii()));
fs.writeFileSync(`${distFolder}/_shadows.scss`, generateVariablesImport(generateShadows()));
fs.writeFileSync(`${distFolder}/_spacing.scss`, generateVariablesImport(generateSpacing()));
fs.writeFileSync(`${distFolder}/_transition.scss`, generateVariablesImport(generateTransition()));
fs.writeFileSync(`${distFolder}/index.scss`, generateVariablesImport(generateImports()));
